@@ -40,8 +40,6 @@ from .const import (
     AMAZON_DEVICE_TYPE,
     BIN_EXTENSION,
     CSRF_COOKIE,
-    DEFAULT_AGENT,
-    DEFAULT_ASSOC_HANDLE,
     DEFAULT_HEADERS,
     DEFAULT_SITE,
     DEVICE_TO_IGNORE,
@@ -55,6 +53,7 @@ from .const import (
     SAVE_PATH,
     SENSORS,
     URI_DEVICES,
+    URI_DND,
     URI_NEXUS_GRAPHQL,
     URI_SIGNIN,
 )
@@ -65,7 +64,7 @@ from .exceptions import (
     CannotRetrieveData,
     WrongMethod,
 )
-from .query import QUERY_DEVICE_STATE
+from .query import QUERY_DEVICE_DATA, QUERY_SENSOR_STATE
 from .utils import obfuscate_email, scrub_fields
 
 
@@ -76,6 +75,8 @@ class AmazonDeviceSensor:
     name: str
     value: str | int | float
     error: bool
+    error_type: str | None
+    error_msg: str | None
     scale: str | None
 
 
@@ -88,6 +89,7 @@ class AmazonDevice:
     device_family: str
     device_type: str
     device_owner_customer_id: str
+    household_device: bool
     device_cluster_members: list[str]
     online: bool
     serial_number: str
@@ -138,11 +140,16 @@ class AmazonEchoApi:
         self._save_raw_data = False
         self._login_stored_data = login_data or {}
         self._serial = self._serial_number()
+        self._account_owner_customer_id: str | None = None
         self._list_for_clusters: dict[str, str] = {}
 
         self._session = client_session
-        self._devices: dict[str, Any] = {}
-        self._sensors_available: bool = True
+        self._final_devices: dict[str, AmazonDevice] = {}
+        self._endpoints: dict[str, str] = {}  # endpoint ID to serial number map
+
+        initial_time = datetime.now(UTC) - timedelta(days=2)  # force initial refresh
+        self._last_devices_refresh: datetime = initial_time
+        self._last_endpoint_refresh: datetime = initial_time
 
         _LOGGER.debug("Initialize library v%s", __version__)
 
@@ -255,9 +262,9 @@ class AmazonEchoApi:
         oauth_params = {
             "openid.return_to": "https://www.amazon.com/ap/maplanding",
             "openid.oa2.code_challenge_method": "S256",
-            "openid.assoc_handle": DEFAULT_ASSOC_HANDLE,
+            "openid.assoc_handle": "amzn_dp_project_dee_ios",
             "openid.identity": "http://specs.openid.net/auth/2.0/identifier_select",
-            "pageId": DEFAULT_ASSOC_HANDLE,
+            "pageId": "amzn_dp_project_dee_ios",
             "accountStatusPolicy": "P1",
             "openid.claimed_id": "http://specs.openid.net/auth/2.0/identifier_select",
             "openid.mode": "checkid_setup",
@@ -338,9 +345,8 @@ class AmazonEchoApi:
         self,
         method: str,
         url: str,
-        input_data: dict[str, Any] | None = None,
+        input_data: dict[str, Any] | list[dict[str, Any]] | None = None,
         json_data: bool = False,
-        amazon_user_agent: bool = True,
     ) -> tuple[BeautifulSoup, ClientResponse]:
         """Return request response context data."""
         _LOGGER.debug(
@@ -353,9 +359,7 @@ class AmazonEchoApi:
 
         headers = DEFAULT_HEADERS.copy()
         headers.update({"Accept-Language": self._language})
-        if not amazon_user_agent:
-            _LOGGER.debug("Changing User-Agent to %s", DEFAULT_AGENT)
-            headers.update({"User-Agent": DEFAULT_AGENT})
+
         if self._csrf_cookie:
             csrf = {CSRF_COOKIE: self._csrf_cookie}
             _LOGGER.debug("Adding to headers: %s", csrf)
@@ -371,14 +375,6 @@ class AmazonEchoApi:
         )
         self._session.cookie_jar.update_cookies(_cookies, URL(f"amazon.{self._domain}"))
 
-        if url.endswith("/auth/token"):
-            headers.update(
-                {
-                    "Content-Type": "application/x-www-form-urlencoded",
-                    "x-amzn-identity-auth-domain": "api.amazon.com",
-                }
-            )
-
         resp: ClientResponse | None = None
         for delay in [0, 1, 2, 5, 8, 12, 21]:
             if delay:
@@ -445,7 +441,7 @@ class AmazonEchoApi:
             mimetypes.guess_extension(content_type.split(";")[0]) or ".raw",
         )
 
-        return BeautifulSoup(await resp.read(), "html.parser"), resp
+        return BeautifulSoup(await resp.read() or "", "html.parser"), resp
 
     async def _save_to_file(
         self,
@@ -585,17 +581,21 @@ class AmazonEchoApi:
         _LOGGER.info("Register device: %s", scrub_fields(login_data))
         return login_data
 
-    async def _get_devices_state(
-        self,
-    ) -> dict[str, Any]:
-        """Get Device State."""
-        payload = {
-            "operationName": "getDevicesState",
-            "variables": {
-                "latencyTolerance": "LOW",
-            },
-            "query": QUERY_DEVICE_STATE,
-        }
+    async def _get_sensors_state(
+        self, endpoint_id_list: list[str]
+    ) -> dict[str, Any] | list[dict[str, Any]]:
+        """Get sensor State."""
+        payload = [
+            {
+                "operationName": "getEndpointState",
+                "variables": {
+                    "endpointId": endpoint_id,
+                    "latencyTolerance": "LOW",
+                },
+                "query": QUERY_SENSOR_STATE,
+            }
+            for endpoint_id in endpoint_id_list
+        ]
 
         _, raw_resp = await self._session_request(
             method=HTTPMethod.POST,
@@ -606,64 +606,150 @@ class AmazonEchoApi:
 
         return await self._response_to_json(raw_resp)
 
-    async def _get_sensors_states(
-        self,
-    ) -> tuple[dict[str, dict[str, Any]], dict[str, dict[str, AmazonDeviceSensor]]]:
+    async def _get_sensors_states(self) -> dict[str, dict[str, AmazonDeviceSensor]]:
         """Retrieve devices sensors states."""
-        devices_state = await self._get_devices_state()
         devices_sensors: dict[str, dict[str, AmazonDeviceSensor]] = {}
-        devices_endpoints: dict[str, dict[str, Any]] = {}
 
-        endpoints = devices_state["data"]["listEndpoints"]
-        for endpoint in endpoints.get("endpoints"):
-            serial_number = (
-                endpoint["serialNumber"]["value"]["text"]
-                if endpoint["serialNumber"]
-                else None
-            )
-            if serial_number in self._devices:
-                devices_sensors[serial_number] = self._get_device_sensor_state(endpoint)
-                devices_endpoints[serial_number] = endpoint
+        # batch endpoints into groups of 3 to reduce number of requests
+        endpoint_ids = list(self._endpoints.keys())
+        batches = [endpoint_ids[i : i + 3] for i in range(0, len(endpoint_ids), 3)]
+        for endpoint_id_batch in batches:
+            sensors_state = await self._get_sensors_state(endpoint_id_batch)
+            _LOGGER.debug("Sensor data - %s", sensors_state)
 
-        return devices_endpoints, devices_sensors
+            if not isinstance(sensors_state, list) and (
+                error := sensors_state.get("errors")
+            ):
+                if isinstance(error, list):
+                    error = error[0]
+                msg = error.get("message", "Unknown error")
+                path = error.get("path", "Unknown path")
+                _LOGGER.error(
+                    "Error retrieving devices state: %s for path %s", msg, path
+                )
+                return {}
+
+            for endpoint_data in sensors_state:
+                if (
+                    not isinstance(endpoint_data, dict)
+                    or not (data := endpoint_data.get("data"))
+                    or not (endpoint := data.get("endpoint"))
+                ):
+                    _LOGGER.error(
+                        "Malformed sensor state data received: %s", endpoint_data
+                    )
+                    return {}
+                serial_number = self._endpoints[endpoint.get("endpointId")]
+
+                if serial_number in self._final_devices:
+                    devices_sensors[serial_number] = self._get_device_sensor_state(
+                        endpoint, serial_number
+                    )
+
+        return devices_sensors
 
     def _get_device_sensor_state(
-        self, endpoint: dict[str, Any]
+        self, endpoint: dict[str, Any], serial_number: str
     ) -> dict[str, AmazonDeviceSensor]:
         device_sensors: dict[str, AmazonDeviceSensor] = {}
-        if endpoint_dnd := endpoint.get("settings", {}).get("doNotDisturb"):
-            device_sensors["dnd"] = AmazonDeviceSensor(
-                name="dnd",
-                value=endpoint_dnd.get("toggleValue"),
-                error=bool(endpoint_dnd.get("error")),
-                scale=None,
-            )
         for feature in endpoint.get("features", {}):
-            if (sensor := SENSORS.get(feature["name"])) is None:
+            if (sensor_template := SENSORS.get(feature["name"])) is None:
                 # Skip sensors that are not in the predefined list
                 continue
 
-            if not (name := sensor["name"]):
+            if not (name := sensor_template["name"]):
                 raise CannotRetrieveData("Unable to read sensor template")
 
             for feature_property in feature.get("properties"):
-                if sensor["name"] != feature_property.get("name"):
+                if sensor_template["name"] != feature_property.get("name"):
                     continue
 
-                value = feature_property[sensor["key"]]
-                scale = value["scale"] if sensor["scale"] else None
-                error = bool(sensor.get("error"))
-                if subkey := sensor["subkey"]:
-                    value = value[subkey]
+                value: str | int | float = "n/a"
+                scale: str | None = None
+
+                # "error" can be None, missing, or a dict
+                api_error = feature_property.get("error") or {}
+                error = bool(api_error)
+                error_type = api_error.get("type")
+                error_msg = api_error.get("message")
+                if not error:
+                    try:
+                        value_raw = feature_property[sensor_template["key"]]
+                        if not value_raw:
+                            _LOGGER.warning(
+                                "Sensor %s [device %s] ignored due to empty value",
+                                name,
+                                serial_number,
+                            )
+                            continue
+                        scale = (
+                            value_raw[scale_template]
+                            if (scale_template := sensor_template["scale"])
+                            else None
+                        )
+                        value = (
+                            value_raw[subkey_template]
+                            if (subkey_template := sensor_template["subkey"])
+                            else value_raw
+                        )
+
+                    except (KeyError, ValueError) as exc:
+                        _LOGGER.warning(
+                            "Sensor %s [device %s] ignored due to errors in feature %s: %s",  # noqa: E501
+                            name,
+                            serial_number,
+                            feature_property,
+                            repr(exc),
+                        )
+                if error:
+                    _LOGGER.debug(
+                        "error in sensor %s - %s - %s", name, error_type, error_msg
+                    )
                 device_sensors[name] = AmazonDeviceSensor(
                     name,
                     value,
                     error,
+                    error_type,
+                    error_msg,
                     scale,
                 )
 
         return device_sensors
 
+    async def _get_devices_endpoint_data(self) -> dict[str, dict[str, Any]]:
+        """Get Devices endpoint data."""
+        payload = {
+            "operationName": "getDevicesBaseData",
+            "query": QUERY_DEVICE_DATA,
+        }
+
+        _, raw_resp = await self._session_request(
+            method=HTTPMethod.POST,
+            url=f"https://alexa.amazon.{self._domain}{URI_NEXUS_GRAPHQL}",
+            input_data=payload,
+            json_data=True,
+        )
+
+        endpoint_data = await self._response_to_json(raw_resp)
+
+        if not (data := endpoint_data.get("data")) or not data.get("listEndpoints"):
+            _LOGGER.error("Malformed endpoint data received: %s", endpoint_data)
+            return {}
+
+        endpoints = data["listEndpoints"]
+        devices_endpoints: dict[str, dict[str, Any]] = {}
+        for endpoint in endpoints.get("endpoints"):
+            # save looking up sensor data on apps
+            if endpoint.get("alexaEnabledMetadata", {}).get("category") == "APP":
+                continue
+
+            if endpoint.get("serialNumber"):
+                serial_number = endpoint["serialNumber"]["value"]["text"]
+                devices_endpoints[serial_number] = endpoint
+                self._endpoints[endpoint["endpointId"]] = serial_number
+
+        return devices_endpoints
+
     async def _response_to_json(self, raw_resp: ClientResponse) -> dict[str, Any]:
         """Convert response to JSON, if possible."""
         try:
@@ -814,11 +900,85 @@ class AmazonEchoApi:
             self._country_specific_data(user_domain)
             await self._refresh_auth_cookies()
 
+    async def _get_account_owner_customer_id(self, data: dict[str, Any]) -> str | None:
+        """Get account owner customer ID."""
+        if data["deviceType"] != AMAZON_DEVICE_TYPE:
+            return None
+
+        account_owner_customer_id: str | None = None
+
+        this_device_serial = self._login_stored_data["device_info"][
+            "device_serial_number"
+        ]
+
+        for subdevice in data["appDeviceList"]:
+            if subdevice["serialNumber"] == this_device_serial:
+                account_owner_customer_id = data["deviceOwnerCustomerId"]
+                _LOGGER.debug(
+                    "Setting account owner: %s",
+                    account_owner_customer_id,
+                )
+                break
+
+        return account_owner_customer_id
+
     async def get_devices_data(
         self,
     ) -> dict[str, AmazonDevice]:
         """Get Amazon devices data."""
-        self._devices = {}
+        delta_devices = datetime.now(UTC) - self._last_devices_refresh
+        if delta_devices >= timedelta(days=1):
+            _LOGGER.debug(
+                "Refreshing devices data after %s",
+                str(timedelta(minutes=round(delta_devices.total_seconds() / 60))),
+            )
+            # Request base device data
+            await self._get_base_devices()
+            self._last_devices_refresh = datetime.now(UTC)
+
+        delta_endpoints = datetime.now(UTC) - self._last_endpoint_refresh
+        if delta_endpoints >= timedelta(minutes=30):
+            _LOGGER.debug(
+                "Refreshing endpoint data after %s",
+                str(timedelta(minutes=round(delta_endpoints.total_seconds() / 60))),
+            )
+            # Set device endpoint data
+            await self._set_device_endpoints_data()
+            self._last_endpoint_refresh = datetime.now(UTC)
+
+        await self._get_sensor_data()
+
+        return self._final_devices
+
+    async def _get_sensor_data(self) -> None:
+        devices_sensors = await self._get_sensors_states()
+        dnd_sensors = await self._get_dnd_status()
+        for device in self._final_devices.values():
+            # Update sensors
+            sensors = devices_sensors.get(device.serial_number, {})
+            if sensors:
+                device.sensors = sensors
+            else:
+                for device_sensor in device.sensors.values():
+                    device_sensor.error = True
+            if device_dnd := dnd_sensors.get(device.serial_number):
+                device.sensors["dnd"] = device_dnd
+
+    async def _set_device_endpoints_data(self) -> None:
+        devices_endpoints = await self._get_devices_endpoint_data()
+        for serial_number in self._final_devices:
+            device_endpoint = devices_endpoints.get(serial_number, {})
+            endpoint_device = self._final_devices[serial_number]
+            endpoint_device.entity_id = (
+                device_endpoint["legacyIdentifiers"]["chrsIdentifier"]["entityId"]
+                if device_endpoint
+                else None
+            )
+            endpoint_device.endpoint_id = (
+                device_endpoint["endpointId"] if device_endpoint else None
+            )
+
+    async def _get_base_devices(self) -> None:
         _, raw_resp = await self._session_request(
             method=HTTPMethod.GET,
             url=f"https://alexa.amazon.{self._domain}{URI_DEVICES}",
@@ -830,20 +990,26 @@ class AmazonEchoApi:
 
         for data in json_data["devices"]:
             dev_serial = data.get("serialNumber")
-            self._devices[dev_serial] = data
+            if not dev_serial:
+                _LOGGER.warning(
+                    "Skipping device without serial number: %s", data["accountName"]
+                )
+                continue
+            if not self._account_owner_customer_id:
+                self._account_owner_customer_id = (
+                    await self._get_account_owner_customer_id(data)
+                )
 
-        devices_endpoints, devices_sensors = await self._get_sensors_states()
+        if not self._account_owner_customer_id:
+            raise CannotRetrieveData("Cannot find account owner customer ID")
 
         final_devices_list: dict[str, AmazonDevice] = {}
-        for device in self._devices.values():
+        for device in json_data["devices"]:
             # Remove stale, orphaned and virtual devices
             if not device or (device.get("deviceType") in DEVICE_TO_IGNORE):
                 continue
 
             serial_number: str = device["serialNumber"]
-            # Add sensors
-            sensors = devices_sensors.get(serial_number, {})
-            device_endpoint = devices_endpoints.get(serial_number, {})
 
             final_devices_list[serial_number] = AmazonDevice(
                 account_name=device["accountName"],
@@ -851,17 +1017,15 @@ class AmazonEchoApi:
                 device_family=device["deviceFamily"],
                 device_type=device["deviceType"],
                 device_owner_customer_id=device["deviceOwnerCustomerId"],
+                household_device=device["deviceOwnerCustomerId"]
+                == self._account_owner_customer_id,
                 device_cluster_members=(device["clusterMembers"] or [serial_number]),
                 online=device["online"],
                 serial_number=serial_number,
                 software_version=device["softwareVersion"],
-                entity_id=device_endpoint["legacyIdentifiers"]["chrsIdentifier"][
-                    "entityId"
-                ]
-                if device_endpoint
-                else None,
-                endpoint_id=device_endpoint["endpointId"] if device_endpoint else None,
-                sensors=sensors,
+                entity_id=None,
+                endpoint_id=None,
+                sensors={},
             )
 
         self._list_for_clusters.update(
@@ -871,7 +1035,7 @@ class AmazonEchoApi:
             }
         )
 
-        return final_devices_list
+        self._final_devices = final_devices_list
 
     async def auth_check_status(self) -> bool:
         """Check AUTH status."""
@@ -925,7 +1089,7 @@ class AmazonEchoApi:
             "deviceType": device.device_type,
             "deviceSerialNumber": device.serial_number,
             "locale": self._language,
-            "customerId": device.device_owner_customer_id,
+            "customerId": self._account_owner_customer_id,
         }
 
         payload: dict[str, Any]
@@ -934,7 +1098,7 @@ class AmazonEchoApi:
                 **base_payload,
                 "textToSpeak": message_body,
                 "target": {
-                    "customerId": device.device_owner_customer_id,
+                    "customerId": self._account_owner_customer_id,
                     "devices": [
                         {
                             "deviceSerialNumber": device.serial_number,
@@ -971,7 +1135,7 @@ class AmazonEchoApi:
                     }
                 ],
                 "target": {
-                    "customerId": device.device_owner_customer_id,
+                    "customerId": self._account_owner_customer_id,
                     "devices": playback_devices,
                 },
                 "skillId": "amzn1.ask.1p.routines.messaging",
@@ -1143,9 +1307,11 @@ class AmazonEchoApi:
             "domain": f"www.amazon.{self._domain}",
         }
 
-        raw_resp = await self._session.post(
+        _, raw_resp = await self._session_request(
+            HTTPMethod.POST,
             "https://api.amazon.com/auth/token",
-            data=data,
+            input_data=data,
+            json_data=False,
         )
         _LOGGER.debug(
             "Refresh data response %s with payload %s",
@@ -1174,3 +1340,24 @@ class AmazonEchoApi:
 
         _LOGGER.debug("Unexpected refresh data response")
         return False, {}
+
+    async def _get_dnd_status(self) -> dict[str, AmazonDeviceSensor]:
+        dnd_status: dict[str, AmazonDeviceSensor] = {}
+        _, raw_resp = await self._session_request(
+            method=HTTPMethod.GET,
+            url=f"https://alexa.amazon.{self._domain}{URI_DND}",
+        )
+
+        dnd_data = await self._response_to_json(raw_resp)
+        _LOGGER.debug("DND data: %s", dnd_data)
+
+        for dnd in dnd_data.get("doNotDisturbDeviceStatusList", {}):
+            dnd_status[dnd.get("deviceSerialNumber")] = AmazonDeviceSensor(
+                name="dnd",
+                value=dnd.get("enabled"),
+                error=False,
+                error_type=None,
+                error_msg=None,
+                scale=None,
+            )
+        return dnd_status
