@@ -4,8 +4,6 @@ import logging
 
 _LOGGER = logging.getLogger(__package__)
 
-DEFAULT_ASSOC_HANDLE = "amzn_dp_project_dee_ios"
-
 HTTP_ERROR_199 = 199
 HTTP_ERROR_299 = 299
 
@@ -48,13 +46,13 @@ DEFAULT_HEADERS = {
     "Accept-Encoding": "gzip",
     "Connection": "keep-alive",
 }
-DEFAULT_AGENT = "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/138.0.0.0 Safari/537.36 Edg/138.0.0.0"  # noqa: E501
 CSRF_COOKIE = "csrf"
 
 REFRESH_ACCESS_TOKEN = "access_token"  # noqa: S105
 REFRESH_AUTH_COOKIES = "auth_cookies"
 
 URI_DEVICES = "/api/devices-v2/device"
+URI_DND = "/api/dnd/device-status-list"
 URI_SIGNIN = "/ap/signin"
 URI_NEXUS_GRAPHQL = "/nexus/v1/graphql"
 
@@ -88,13 +86,28 @@ SENSORS: dict[str, dict[str, str | None]] = {
         "subkey": "value",
         "scale": None,
     },
-    "speaker": {
-        "name": "volume",
-        "key": "value",
-        "subkey": "volValue",
-        "scale": None,
-    },
 }
+
+ALEXA_INFO_SKILLS = [
+    "Alexa.Calendar.PlayToday",
+    "Alexa.Calendar.PlayTomorrow",
+    "Alexa.Calendar.PlayNext",
+    "Alexa.Date.Play",
+    "Alexa.Time.Play",
+    "Alexa.News.NationalNews",
+    "Alexa.FlashBriefing.Play",
+    "Alexa.Traffic.Play",
+    "Alexa.Weather.Play",
+    "Alexa.CleanUp.Play",
+    "Alexa.GoodMorning.Play",
+    "Alexa.SingASong.Play",
+    "Alexa.FunFact.Play",
+    "Alexa.Joke.Play",
+    "Alexa.TellStory.Play",
+    "Alexa.ImHome.Play",
+    "Alexa.GoodNight.Play",
+]
+
 DEVICE_TO_IGNORE: list[str] = [
     AMAZON_DEVICE_TYPE,  # Alexa App for iOS
     "A2TF17PFR55MTB",  # Alexa App for Android
@@ -116,7 +129,9 @@ DEVICE_TO_IGNORE: list[str] = [
     "AYHO3NTIQQ04G",  # Nextbase 622GW Dash Cam - issue #477
     "AHL4H6CKH3AUP",  # BMW Car System - issue #478
     "A3BW5ZVFHRCQPO",  # BMW Mini Car System - issue #479
-    "A1M0A9L9HDBID3",  # Sony Soundbar Sony HT-A5000 - issue #486
+    "A133UZ2CB0IB8",  # Sony Soundbar Sony HT-A5000 - issue #486
+    "A2M9HB23M9MSSM",  # Smartwatch Amazfit Bip U Pro - issue #507
+    "A1P7E7V3FCZKU6",  # Toshiba Corporation TV 32LF221U19 - issue #531
 ]
 
 DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
@@ -148,6 +163,11 @@ DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
         "model": "Echo Show 15",
         "hw_version": "Gen1",
     },
+    "A1M0A9L9HDBID3": {
+        "manufacturer": "First Alert",
+        "model": "Onelink Smoke + Carbon Monoxide Alarm",
+        "hw_version": None,
+    },
     "A1NL4BVLQ4L3N3": {
         "model": "Echo Show",
         "hw_version": "Gen1",
@@ -194,6 +214,15 @@ DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
         "model": "Echo Show 8",
         "hw_version": "Gen1",
     },
+    "A1ZOM3H3U20BL8": {
+        "manufacturer": "Medion",
+        "model": "Life P66970",
+        "hw_version": "A16",
+    },
+    "A25521KS9QCAMD": {
+        "model": "Fire Tablet HD 7",
+        "hw_version": "Gen4",
+    },
     "A265XOI9586NML": {
         "model": "Fire TV Stick with Alexa Voice Remote",
         "hw_version": None,
@@ -369,6 +398,11 @@ DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
         "model": "Echo",
         "hw_version": "Gen2",
     },
+    "AD2YUJTRVBNOF": {
+        "manufacturer": "Sony Group Corporation",
+        "model": "HT-Z9F",
+        "hw_version": None,
+    },
     "ADMKNMEVNL158": {
         "model": "Echo Hub",
         "hw_version": "Gen1",
@@ -414,6 +448,11 @@ DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
         "model": "Fire Tablet HD 10",
         "hw_version": "Gen11",
     },
+    "AU3ZY84EWI70R": {
+        "manufacturer": "Medion",
+        "model": "Life P61142",
+        "hw_version": "A16",
+    },
     "AUPUQSVCVHXP0": {
         "manufacturer": "ecobee Inc.",
         "model": "ecobee Switch+",
@@ -436,29 +475,4 @@ DEVICE_TYPE_TO_MODEL: dict[str, dict[str, str | None]] = {
         "model": "Echo Plus",
         "hw_version": "Gen2",
     },
-    "A1M0A9L9HDBID3": {
-        "manufacturer": "First Alert",
-        "model": "Onelink Smoke + Carbon Monoxide Alarm",
-        "hw_version": None,
-    },
 }
-
-ALEXA_INFO_SKILLS = [
-    "Alexa.Calendar.PlayToday",
-    "Alexa.Calendar.PlayTomorrow",
-    "Alexa.Calendar.PlayNext",
-    "Alexa.Date.Play",
-    "Alexa.Time.Play",
-    "Alexa.News.NationalNews",
-    "Alexa.FlashBriefing.Play",
-    "Alexa.Traffic.Play",
-    "Alexa.Weather.Play",
-    "Alexa.CleanUp.Play",
-    "Alexa.GoodMorning.Play",
-    "Alexa.SingASong.Play",
-    "Alexa.FunFact.Play",
-    "Alexa.Joke.Play",
-    "Alexa.TellStory.Play",
-    "Alexa.ImHome.Play",
-    "Alexa.GoodNight.Play",
-]
